

import concurrent.futures as cf
import os
import traceback
import time
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from numba import njit
import dask.array as da

def surf(p, q, gp, gq, data, resid, weight, flag, basename):
    """
    This here is to get the stack trace when an exception is thrown
    """
    try:
        return _surf(p, q, gp, gq, data, resid, weight, flag, basename)
    except Exception as e:
        traceback_str = traceback.format_exc(e)
        raise StandardError("Error occurred. Original traceback "
                            "is\n%s\n" % traceback_str)

def surfchisq(resid, weight, flag, ant1, ant2,
          rbin_idx, rbin_counts, fbin_idx, fbin_counts):

    nant = da.maximum(ant1.max(), ant2.max()).compute() + 1
    res = da.blockwise(_surfchisq, 'tfcpq2',
                       resid, 'tfc',
                       weight, 'tfc',
                       flag, 'tfc',
                       ant1, 't',
                       ant2, 't',
                       rbin_idx, 't',
                       rbin_counts, 't',
                       fbin_idx, 'f',
                       fbin_counts, 'f',
                       align_arrays=False,
                       dtype=np.float64,
                       adjust_chunks={'t': rbin_idx.chunks[0],
                                      'f': fbin_idx.chunks[0]},
                       new_axes={'p': nant, 'q': nant, '2': 2})
    return res


@njit(fastmath=True, nogil=True)
def _surfchisq(resid, weight, flag, ant1, ant2,
           rbin_idx, rbin_counts, fbin_idx, fbin_counts):
    nrow, nchan, ncorr = resid.shape

    nto = rbin_idx.size
    nfo = fbin_idx.size
    uant1 = np.unique(ant1)
    uant2 = np.unique(ant2)
    nant = np.maximum(uant1.max(), uant2.max()) + 1

    # init output array
    out = np.zeros((nto, nfo, ncorr, nant, nant, 2), dtype=np.float64)

    # account for chunk indexing
    rbin_idx2 = rbin_idx - rbin_idx.min()
    fbin_idx2 = fbin_idx - fbin_idx.min()
    for t in range(nto):
        rowi = rbin_idx2[t]
        rowf = rbin_idx2[t] + rbin_counts[t]
        residr = resid[rowi:rowf]
        weightr = weight[rowi:rowf]
        flagr = flag[rowi:rowf]
        ant1r = ant1[rowi:rowf]
        ant2r = ant2[rowi:rowf]
        for f in range(nfo):
            chani = fbin_idx2[f]
            chanf = fbin_idx2[f] + fbin_counts[f]
            residrf = residr[:, chani:chanf]
            weightrf = weightr[:, chani:chanf]
            flagrf = flagr[:, chani:chanf]
            for c in range(ncorr):
                residrfc = residrf[:, :, c]
                weightrfc = weightrf[:, :, c]
                flagrfc = flagrf[:, :, c]
                for p in uant1:
                    Ip = ant1r == p
                    for q in uant2:
                        Iq = ant2r == q
                        Ipq = Ip & Iq
                        R = residrfc[Ipq].ravel()
                        W = weightrfc[Ipq].ravel()
                        F = flagrfc[Ipq].ravel()
                        for i in range(R.size):
                            if not F[i] and p != q:
                                out[t, f, c, p, q, 0] += (np.conj(R[i]) * W[i] * R[i]).real
                                out[t, f, c, p, q, 1] += 1.0
                        out[t, f, c, q, p] = out[t, f, c, p, q]

    return out


def flagchisq(resid, weight, flag, use_corrs, sigma=25):

    res = da.blockwise(_flagchisq, 'rfc',
                       resid, 'rfc',
                       weight, 'rfc',
                       flag, 'rfc',
                       use_corrs, None,
                       sigma, None,
                       dtype=bool)
    return res


@njit(fastmath=True, nogil=True)
def _flagchisq(resid, weight, flag, use_corrs, sigma):
    nrow, nchan, ncorr = resid.shape
    sigmasq = sigma**2
    for r in range(nrow):
        for f in range(nchan):
            for c in use_corrs:
                res = resid[r, f, c]
                w = weight[r, f, c]
                chi2 = (np.conj(res) * w * res).real
                if chi2 > sigmasq or chi2 == 0:
                    flag[r, f, c] = True
                else:
                    flag[r, f, c] = False
    return flag



#!/usr/bin/env python
import matplotlib as mpl
mpl.rcParams.update({'font.size': 11, 'font.family': 'serif'})
# mpl.use('Agg')
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from optparse import OptionParser
import os
import sys
import numpy as np
import xarray as xr
import dask
import dask.array as da
from dask.diagnostics import ProgressBar
from surfvis.utils import flagchisq
from daskms import xds_from_ms, xds_from_table, xds_to_table


def create_parser():
    parser = OptionParser(usage='%prog [options] msname')
    parser.add_option('--rcol', default='RESIDUAL',
                      help='Residual column (default = RESIDUAL)')
    parser.add_option('--scol',
                      help='sigma column to use. Overides wcol if provided.')
    parser.add_option('--wcol', default='WEIGHT_SPECTRUM',
                      help='Weight column (default = WEIGHT_SPECTRUM)')
    parser.add_option('--fcol', default='FLAG',
                      help='Flag column (default = FLAG)')
    parser.add_option('--sigma', default=3, type=float,
                      help='chisq threshold (default = 3)')
    parser.add_option('--nthreads', default=4, type=int,
                      help='Number of dask threads to use')
    parser.add_option('--nrows', default=10000, type=int,
                      help='Number of rows in each chunk (default=10000)')
    parser.add_option('--nfreqs', default=128, type=int,
                      help='Number of frequencies in a chunk (default=128)')
    parser.add_option("--use-corrs", type=str,
                      help='Comma seprated list of correlations to use (do not use spaces)')
    return parser

def main():
    (options,args) = create_parser().parse_args()

    # Some error trapping
    if len(args) != 1:
        print('Please specify a single Measurement set to flag.')
        sys.exit(-1)
    else:
        msname = args[0].rstrip('/')

    schema = {}
    schema[options.rcol] = {'dims': ('chan', 'corr')}
    schema[options.fcol] = {'dims': ('chan', 'corr')}
    columns = [options.rcol, options.fcol]
    if options.scol is not None:
        schema[options.scol] = {'dims': ('chan', 'corr')}
        columns.append(options.scol)
    else:
        schema[options.wcol] = {'dims': ('chan', 'corr')}
        columns.append(options.wcol)


    xds = xds_from_ms(msname,
                      columns=columns,
                      chunks={'row': options.nrows, 'chan': options.nfreqs},
                      group_cols=['FIELD_ID', 'DATA_DESC_ID', 'SCAN_NUMBER'],
                      table_schema=schema)

    if options.use_corrs is None:
        print('Using only diagonal correlations')
        if len(xds[0].corr) > 1:
            use_corrs = [0, -1]
        else:
            use_corrs = [0]
    else:
        use_corrs = list(map(int, options.use_corrs.split(',')))
        print(f"Using correlations {use_corrs}")

    out_data = []
    for i, ds in enumerate(xds):
        resid = ds.get(options.rcol).data
        if options.scol is not None:
            sigma = ds.get(options.scol).data
            weight = 1/sigma**2
        else:
            weight = ds.get(options.wcol).data
        flag = ds.get(options.fcol).data

        uflag = flagchisq(resid, weight, flag, tuple(use_corrs), sigma=options.sigma)

        out_ds = ds.assign(**{options.fcol: (("row", "chan", "corr"), uflag)})
        out_data.append(out_ds)

    writes = xds_to_table(out_data, msname, columns=[options.fcol])

    with ProgressBar():
        dask.compute(writes)



#!/usr/bin/env python

import matplotlib
matplotlib.use('Agg')
from optparse import OptionParser
import pylab
import pyrap.tables
import os
import sys
import numpy
from daskms import xds_from_ms


def gi(message):
        print('\033[92m'+message+'\033[0m')


def ri(message):
        print('\033[91m'+message+'\033[0m')


# COMMAND LINE OPTIONS
def create_parser():
	parser = OptionParser(usage='%prog [options] msname')
	#parser.add_option('-m','--msname',dest='msname',help='Measurement Set to open',metavar='DIRECTORY')
	parser.add_option('-l','--list',dest='dolist',action='store_true',help='List Measurement Set properties and exit',default=False)
	parser.add_option('-d','--datacolumn',dest='column',help='Measurement Set column to plot (default = DATA)',default='DATA')
	parser.add_option('-f','--field',dest='field',help='Field ID to plot (default = 0)',default=0)
	parser.add_option('-s','--spw',dest='myspw',help='Comma separated list of SPWs to plot (default = all)',default='')
	parser.add_option('-p','--plot',dest='plot',help='Set to amp, phase, real or imag (default = amp)',default='amp')
	parser.add_option('-i','--i',dest='antenna1',help='Antenna 1: plot only this antenna',default=-1)
	parser.add_option('-j','--j',dest='antenna2',help='Antenna 2: use with -i to plot a single baseline',default=-1)
	parser.add_option('--noflags',dest='noflags',help='Disable flagged data overlay',action='store_true',default=False)
	parser.add_option('--scale',dest='scale',help='Scale the image peak to this multiple of the per-corr min/max (default = scale image max to 5 sigma, this parameter is ignored for phase plots)',default=-1)
	parser.add_option('--cmap',dest='mycmap',help='Matplotlib colour map to use (default = jet)',default='jet')
	parser.add_option('-o','--opdir',dest='foldername',help='Output folder to store plots (default = msname___plots)',default='')
	return parser

def main():
	(options,args) = create_parser().parse_args()

	dolist = options.dolist
	column = options.column
	fieldid = int(options.field)
	myspw = options.myspw
	plot = options.plot
	antenna1 = int(options.antenna1)
	antenna2 = int(options.antenna2)
	noflags = options.noflags
	scale = float(options.scale)
	mycmap = options.mycmap
	foldername = options.foldername

	# Some error trapping

	if len(args) != 1:
		ri('Please specify a single Measurement Set to plot.')
		sys.exit(-1)
	else:
		msname = args[0].rstrip('/')

	if plot not in ['amp','phase','real','imag']:
		ri('Requested plot not valid, must be one of amp, phase, real or imag.')
		sys.exit(-1)


	# MEASUREMENT SET INFO

	fieldtab = pyrap.tables.table(msname+'/FIELD')
	sourceids = fieldtab.getcol('SOURCE_ID')
	sourcenames = fieldtab.getcol('NAME')
	fieldtab.done()

	spwtab = pyrap.tables.table(msname+'/SPECTRAL_WINDOW')
	nspw = len(spwtab)
	spwfreqs = spwtab.getcol('REF_FREQUENCY')
	chanwidth = spwtab.getcol('CHAN_WIDTH')[0][0] # probably needs changing if SPWs have different widths
	nchans = spwtab.getcol('NUM_CHAN')
	spwtab.done()

	anttab = pyrap.tables.table(msname+'/ANTENNA')
	nant = len(anttab)
	antpos = anttab.getcol('POSITION')
	antnames = anttab.getcol('NAME')
	anttab.done()

	tt = pyrap.tables.table(msname)
	usedants = numpy.unique(tt.getcol('ANTENNA1'))


	# PRINT SUMMARY

	if dolist:
		print('')
		gi('     '+msname+'/FIELD')
		gi('     ROW   ID            NAME')
		for i in range(0,len(sourceids)):
			print('     %-6s%-14s%-14s' % (i,sourceids[i],sourcenames[i]))
		print('')
		gi('     '+msname+'/SPECTRAL_WINDOW')
		gi('     ROW   CHANS         WIDTH[MHz]          REF_FREQ[MHz]')
		for i in range(0,nspw):
			print('     %-6s%-14s%-20s%-14s' % (i,str(nchans[i]),str(chanwidth/1e6),str(spwfreqs[i]/1e6)))
		print('')
		gi('     '+msname+'/ANTENNA')
		gi('     ROW   NAME          POSITION')
		for i in range(0,nant):
			if i in usedants:
				print('     %-6s%-14s%-14s' % (i,(antnames[i]),str(antpos[i])))
			else:
				ri('     %-6s%-14s%-14s' % (i,(antnames[i]),str(antpos[i])))

		print('')
		tt.done()


	# OR ELSE DO THE PLOTS


	else:
		# Create output folder if it doesn't exist
		if foldername == '':
			foldername = msname+'_'+column+'__plots'
		if os.path.isdir(foldername):
			print('Found',foldername)
		else:
			print('Creating',foldername)
			os.system('mkdir '+foldername)

		# Make a complete list of SPWs if one isn't provided
		if myspw == '':
			myspw = numpy.arange(0,nspw)
		else:
			myspw = myspw.split(',')

		fieldname = sourcenames[fieldid]

		# Make a list of baseline pairs based on the antenna selections
		baselines = []
		if antenna1 != -1 and antenna2 != -1:
			baselines = [(antenna1,antenna2)]
		elif antenna1 != -1:
			i = antenna1
			for j in usedants:
				if i != j:
					pair = [i,j]
					pair = sorted(pair)
					if pair not in baselines:
						if antenna1 != -1:
							if antenna1 in pair:
								baselines.append(pair)
						else:
							baselines.append(pair)
		else:
			for i in usedants:
				for j in usedants:
					if i != j:
						pair = [i,j]
						pair = sorted(pair)
						if pair not in baselines:
							if antenna1 != -1:
								if antenna1 in pair:
									baselines.append(pair)
							else:
								baselines.append(pair)

		# Loop over baselines
		for baseline in baselines:

			# Determine unprojected baseline length
			ap1 = antpos[baseline[0]]
			ap2 = antpos[baseline[1]]
			blength = (((ap1[0]-ap2[0])**2.0)+((ap1[1]-ap2[1])**2.0)+((ap1[2]-ap1[2])**2.0))**0.5
			blength = str(round(blength/1000.0,2))

			print('Plotting baseline:',baseline,'     Deprojected length:',blength,'km')

			# Get the data
			datacols = []
			flagcols = []
			# Loop over SPWs
			print('SPW:')
			for spw in myspw:
				print(spw)
				subtab = tt.query(query='ANTENNA1=='+str(baseline[0])
					+' && ANTENNA2=='+str(baseline[1])
					+' && DATA_DESC_ID=='+str(spw)
					+' && FIELD_ID=='+str(fieldid))
				datacol = subtab.getcol(column)
				flagcol = subtab.getcol('FLAG')
				datacols.append(datacol)
				flagcols.append(flagcol)
			print('')
			# Reshape the data
			baselinedata = datacols[0]
			flagdata = flagcols[0]
			for p in range(1,len(datacols)):
				baselinedata = numpy.concatenate((baselinedata,datacols[p]),axis=1)
				flagdata = numpy.concatenate((flagdata,flagcols[p]),axis=1)

			# Get number of corr products from the data shape
			n_corr = baselinedata.shape[2]

			# Generate png name
			pngname = foldername+'/'+msname.split('/')[-1].rstrip('/')+'_baseline_'+str(baseline[0])+'_'+str(baseline[1])
			pngname+='_field'+str(fieldid)
			pngname+='_'+plot+'.png'

			# Generate figure title
			figtitle = 'MS: '+msname.rstrip('/')
			figtitle += '\nColumn: '+column+', '+plot
			figtitle += '\nBaseline: '+str(baseline[0])+'-'+str(baseline[1])
			figtitle += ' ['+blength+' km]'
			figtitle+='\nField: '+fieldname

			# Create the figure
			fig = pylab.figure(figsize=(20,15))
			t = fig.text(0.5, 0.945, figtitle,horizontalalignment='center',color='blue')

			# A panel for each corr product
			for k in range(0,n_corr):
				if plot == 'phase':
					plotdata = numpy.angle(baselinedata[:,:,k]) # radians
				elif plot == 'real':
					plotdata = baselinedata[:,:,k].real
				elif plot == 'imag':
					plotdata = baselinedata[:,:,k].imag
				else:
					plotdata = numpy.absolute(baselinedata[:,:,k])

				flagimage = pylab.cm.gray(plotdata*0.0)
				flagimage[:,:,3] = (flagdata[:,:,k])

				ax = fig.add_subplot(1,n_corr,k+1)
				ax.set_xlabel('Channel number')
				if k==0:
					ax.set_ylabel('Time slot')
				elif k==n_corr-1:
					ax.yaxis.tick_right()
					ax.yaxis.set_label_position('right')
					ax.set_ylabel('Time slot')
				else:
					for ytick_i in ax.get_yticklabels():
						ytick_i.set_visible(False)
				if plot != 'phase' and len(plotdata)>0:
					if scale != -1:
						immax = scale*plotdata.max()
						immin = scale*plotdata.min()
					else:
						imstd = numpy.std(plotdata)
						immax = 5.0*imstd
						immin = 0.0
					ax.imshow(plotdata,aspect='auto',clim=(immin,immax),cmap=mycmap)
					if not noflags:
						ax.imshow(flagimage,aspect='auto',interpolation='nearest')
				elif len(plotdata)>0:
					ax.imshow(plotdata,aspect='auto',cmap=mycmap)
					if not noflags:
						ax.imshow(flagimage,aspect='auto',interpolation='nearest')
				else:
					ax.imshow(((0,0),(0,0)),aspect='auto')

				ax.set_title('Corr product '+str(k))

				print('    Corr product:',k,'      Data min,max:',plotdata.min(),plotdata.max())

			for o in fig.findobj(matplotlib.text.Text):
				o.set_fontsize('11')

			fig.tight_layout(w_pad=0.98,h_pad=0.98,rect=[0.02,0.02,0.95,0.95])

			pylab.savefig(pngname)
			pylab.close()



#!/usr/bin/env python
import matplotlib as mpl
mpl.rcParams.update({'font.size': 11, 'font.family': 'serif'})
# mpl.use('Agg')
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from optparse import OptionParser
import os
import sys
import numpy as np
import xarray as xr
import dask
import dask.array as da
from dask.diagnostics import ProgressBar
from surfvis.utils import surfchisq
from daskms import xds_from_ms, xds_from_table
from daskms.experimental.zarr import xds_from_zarr, xds_to_zarr
# might make for cooler histograms but doesn't work out of the box
from astropy.visualization import hist


# COMMAND LINE OPTIONS
def create_parser():
	parser = OptionParser(usage='%prog [options] msname')
	parser.add_option('--rcol', default='RESIDUAL',
					  help='Residual column (default = RESIDUAL)')
	parser.add_option('--wcol', default='WEIGHT_SPECTRUM',
					  help='Weight column (default = WEIGHT_SPECTRUM)')
	parser.add_option('--fcol', default='FLAG',
					  help='Flag column (default = FLAG)')
	parser.add_option('--dataout', default='',
					  help='Output name of zarr dataset')
	parser.add_option('--imagesout', default=None,
					  help='Output folder to place images. '
					  	   'If None (default) no plots are saved')
	parser.add_option('--nthreads', default=4, type=int,
					  help='Number of dask threads to use')
	parser.add_option('--ntimes', default=-1, type=int,
				      help='Number of unique times in each chunk.')
	parser.add_option('--nfreqs', default=128, type=int,
					  help='Number of frequencies in a chunk.')
	parser.add_option("--use-corrs", type=str,
					  help='Comma seprated list of correlations to use (do '
					  'not use spaces). Default = diagonal correlations')
	return parser

def main():
	(options,args) = create_parser().parse_args()

	if options.dataout == '':
		options.dataout = os.getcwd() + '/chi2'

	if os.path.isdir(options.dataout):
		print(f"Removing existing {options.dataout} folder")
		os.system(f"rm -r {options.dataout}")

	# Some error trapping
	if len(args) != 1:
		print('Please specify a single Measurement Set to plot.')
		sys.exit(-1)
	else:
		msname = args[0].rstrip('/')

	# chunking info
	schema = {}
	schema[options.fcol] = {'dims': ('chan', 'corr')}
	xds = xds_from_ms(msname,
					  chunks={'row': -1},
					  columns=['TIME', options.fcol],
					  group_cols=['FIELD_ID', 'DATA_DESC_ID', 'SCAN_NUMBER'],
					  table_schema=schema)

	chunks = []
	rbin_idx = []
	rbin_counts = []
	tbin_idx = []
	tbin_counts = []
	fbin_idx = []
	fbin_counts = []
	t0s = []
	tfs = []
	for ds in xds:
		time = ds.TIME.values
		ut, counts = np.unique(time, return_counts=True)
		if options.ntimes in [0, -1]:
			utpc = ut.size
		else:
			utpc = options.ntimes
		row_chunks = [np.sum(counts[i:i+utpc])
                 	  for i in range(0, ut.size, utpc)]

		nchan = ds.chan.size
		if options.nfreqs in [0, -1]:
			options.nfreqs = nchan

		# list per ds
		chunks.append({'row': tuple(row_chunks), 'chan': options.nfreqs})

		ridx = np.zeros(len(row_chunks))
		ridx[1:] = np.cumsum(row_chunks)[0:-1]
		rbin_idx.append(da.from_array(ridx.astype(int), chunks=1))
		rbin_counts.append(da.from_array(row_chunks, chunks=1))

		ntime = ut.size
		tidx = np.arange(0, ntime, utpc)
		tbin_idx.append(da.from_array(tidx.astype(int), chunks=1))
		tidx2 = np.append(tidx, ntime)
		tcounts = tidx2[1:] - tidx2[0:-1]
		tbin_counts.append(da.from_array(tcounts, chunks=1))

		t0 = ut[tidx]
		t0s.append(da.from_array(t0, chunks=1))
		tf = ut[tidx + tcounts -1]
		tfs.append(da.from_array(tf, chunks=1))

		fidx = np.arange(0, nchan, options.nfreqs)
		fbin_idx.append(da.from_array(fidx, chunks=1))
		fidx2 = np.append(fidx, nchan)
		fcounts = fidx2[1:] - fidx2[0:-1]
		fbin_counts.append(da.from_array(fcounts, chunks=1))

	schema = {}
	schema[options.rcol] = {'dims': ('chan', 'corr')}
	schema[options.wcol] = {'dims': ('chan', 'corr')}
	schema[options.fcol] = {'dims': ('chan', 'corr')}

	xds = xds_from_ms(msname,
						columns=[options.rcol, options.wcol, options.fcol,
								'ANTENNA1', 'ANTENNA2', 'TIME'],
						chunks=chunks,
						group_cols=['FIELD_ID', 'DATA_DESC_ID', 'SCAN_NUMBER'],
						table_schema=schema)

	if options.use_corrs is None:
		print('Using only diagonal correlations')
		if len(xds[0].corr) > 1:
			use_corrs = [0, -1]
		else:
			use_corrs = [0]
	else:
		use_corrs = list(map(int, options.use_corrs.split(',')))
		print(f"Using correlations {use_corrs}")
	ncorr = len(use_corrs)

	out_ds = []
	idts = []
	for i, ds in enumerate(xds):
		ds = ds.sel(corr=use_corrs)

		resid = ds.get(options.rcol).data
		weight = ds.get(options.wcol).data
		flag = ds.get(options.fcol).data
		ant1 = ds.ANTENNA1.data
		ant2 = ds.ANTENNA2.data

		# ncorr = resid.shape[0]

		# time = ds.TIME.values
		# utime = np.unique(time)

		# spw = xds_from_table(msname + '::SPECTRAL_WINDOW')
		# freq = spw[0].CHAN_FREQ.values

		field = ds.FIELD_ID
		ddid = ds.DATA_DESC_ID
		scan = ds.SCAN_NUMBER

		tmp = surfchisq(resid, weight, flag, ant1, ant2,
				        rbin_idx[i], rbin_counts[i],
				        fbin_idx[i], fbin_counts[i])

		d = xr.Dataset(
			data_vars={'data': (('time', 'freq', 'corr', 'p', 'q', '2'), tmp),
					   'fbin_idx': (('freq'), fbin_idx[i]),
					   'fbin_counts': (('freq'), fbin_counts[i]),
					   'tbin_idx': (('time'), tbin_idx[i]),
					   'tbin_counts': (('time'), tbin_counts[i])},
			attrs = {'FIELD_ID': ds.FIELD_ID,
					 'DATA_DESC_ID': ds.DATA_DESC_ID,
					 'SCAN_NUMBER': ds.SCAN_NUMBER},
			# coords={'time': (('time'), utime),
			# 		'freq': (('freq'), freq),
			# 		'corr': (('corr'), np.arange(ncorr))}
		)

		idt = f'::F{ds.FIELD_ID}_D{ds.DATA_DESC_ID}_S{ds.SCAN_NUMBER}'
		out_ds.append(xds_to_zarr(d, options.dataout + idt))
		idts.append(idt)


	with ProgressBar():
		dask.compute(out_ds)

	# primitive plotting
	if options.imagesout is not None:
		foldername = options.imagesout.rstrip('/')
		if not os.path.isdir(foldername):
				os.system('mkdir '+ foldername)

		for idt in idts:
			xds = xds_from_zarr(options.dataout + idt)
			for ds in xds:
				field = ds.FIELD_ID
				if not os.path.isdir(foldername + f'/field{field}'):
					os.system('mkdir '+ foldername + f'/field{field}')

				spw = ds.DATA_DESC_ID
				if not os.path.isdir(foldername + f'/field{field}' + f'/spw{spw}'):
					os.system('mkdir '+ foldername + f'/field{field}' + f'/spw{spw}')

				scan = ds.SCAN_NUMBER
				if not os.path.isdir(foldername + f'/field{field}' + f'/spw{spw}' + f'/scan{scan}'):
					os.system('mkdir '+ foldername + f'/field{field}' + f'/spw{spw}'+ f'/scan{scan}')

				tmp = ds.data.values
				tbin_idx = ds.tbin_idx.values
				tbin_counts = ds.tbin_counts.values
				fbin_idx = ds.fbin_idx.values
				fbin_counts = ds.fbin_counts.values

				ntime, nfreq, ncorr, _, _, _ = tmp.shape

				basename = foldername + f'/field{field}' + f'/spw{spw}'+ f'/scan{scan}/'
				if len(os.listdir(basename)):
					print(f"Removing contents of {basename} folder")
					os.system(f'rm {basename}*.png')
				for t in range(ntime):
					for f in range(nfreq):
						for c in range(ncorr):
							chi2 = tmp[t, f, c, :, :, 0]
							N = tmp[t, f, c, :, :, 1]
							chi2_dof = np.zeros_like(chi2)
							chi2_dof[N>0] = chi2[N>0]/N[N>0]
							chi2_dof[N==0] = np.nan
							t0 = tbin_idx[t]
							tf = tbin_idx[t] + tbin_counts[t]
							chan0 = fbin_idx[f]
							chanf = fbin_idx[f] + fbin_counts[f]
							makeplot(chi2_dof, basename + f't{t}_f{f}_c{c}.png',
									 f't {t0}-{tf}, chan {chan0}-{chanf}, corr {c}')

						# reduce over corr
						chi2 = np.nansum(tmp[t, f, (0, -1), :, :, 0], axis=0)
						N = np.nansum(tmp[t, f, (0, -1), :, :, 1], axis=0)
						chi2_dof = np.zeros_like(chi2)
						chi2_dof[N>0] = chi2[N>0]/N[N>0]
						chi2_dof[N==0] = np.nan
						t0 = tbin_idx[t]
						tf = tbin_idx[t] + tbin_counts[t]
						chan0 = fbin_idx[f]
						chanf = fbin_idx[f] + fbin_counts[f]
						makeplot(chi2_dof, basename + f't{t}_f{f}.png',
								 f't {t0}-{tf}, chan {chan0}-{chanf}')

					# reduce over freq
					chi2 = np.nansum(tmp[t, :, (0, -1), :, :, 0], axis=(0,1))
					N = np.nansum(tmp[t, :, (0, -1), :, :, 1], axis=(0,1))
					chi2_dof = np.zeros_like(chi2)
					chi2_dof[N>0] = chi2[N>0]/N[N>0]
					chi2_dof[N==0] = np.nan
					t0 = tbin_idx[t]
					tf = tbin_idx[t] + tbin_counts[t]
					makeplot(chi2_dof, basename + f't{t}.png',
					         f't {t0}-{tf}')

				# now the entire scan
				chi2 = np.nansum(tmp[:, :, (0, -1), :, :, 0], axis=(0,1,2))
				N = np.nansum(tmp[:, :, (0, -1), :, :, 1], axis=(0,1,2))
				chi2_dof = np.zeros_like(chi2)
				chi2_dof[N>0] = chi2[N>0]/N[N>0]
				chi2_dof[N==0] = np.nan
				makeplot(chi2_dof, basename + f'scan.png',
						 f'scan {scan}.png')

def makeplot(data, name, subt):
	nant, _ = data.shape
	fig = plt.figure()
	ax = plt.gca()
	im = ax.imshow(data, cmap='inferno')
	ax.set_xticks(np.arange(0, nant, 2))
	ax.set_yticks(np.arange(nant))
	ax.tick_params(axis='both', which='major',
					  length=1, width=1, labelsize=4)

	divider = make_axes_locatable(ax)
	cax = divider.append_axes("bottom", size="3%", pad=0.2)
	cb = fig.colorbar(im, cax=cax, orientation="horizontal")
	cb.outline.set_visible(False)
	cb.ax.tick_params(length=1, width=1, labelsize=6, pad=0.1)

	rax = divider.append_axes("right", size="50%", pad=0.025)
	x = data[~ np.isnan(data)]
	hist(x, bins='scott', ax=rax, histtype='stepfilled',
		 alpha=0.5, density=False)
	rax.set_yticks([])
	rax.tick_params(axis='y', which='both',
					bottom=False, top=False,
					labelbottom=False)
	rax.tick_params(axis='x', which='both',
					length=1, width=1, labelsize=8)

	fig.suptitle(subt, fontsize=20)
	plt.savefig(name, dpi=250)
	plt.close(fig)



from setuptools import setup, find_packages
import surfvis

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = [
                'matplotlib',
                'python-casacore',
                'argparse',
                'ipython',
                'codex-africanus[dask]',
                'astropy',
                'scipy',
                'dask-ms[xarray, zarr]',
                'pytest >= 6.2.2',
            ]


setup(
     name='surfvis',
     version=surfvis.__version__,
     author="Ian Heywood",
     author_email="ianh@astro.ox.ac.uk",
     description="Per-baseline time/freq and Chi-Square plots",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/ratt-ru/surfvis",
     packages=find_packages(),
     python_requires='>=3.7',
     install_requires=requirements,
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
     entry_points='''
                    [console_scripts]
                    surfvis=surfvis.surfvis:main
                    surfchi2=surfvis.surfchi2:main
                    flagchi2=surfvis.flagchi2:main
     '''
     ,
 )
